
// Light Radio v2.4 By CrownSoft (c) 2013
// http://www.crownsoft.net

#ifdef _MSC_VER
#define _CRT_SECURE_NO_DEPRECATE
#endif

#include <windows.h>
#include <commctrl.h> 
#include <rpc.h>
#include <process.h>
#include <stdio.h>
#include "bass.h"
#include "bassenc.h"
#include "basswma.h"
#include "resource.h"

#include <Shlobj.h> // for PathCleanupSpec

#ifdef _MSC_VER
#pragma comment(lib,"COMCTL32.LIB") // for trackbar
#pragma comment(lib,"bass.lib")
#pragma comment(lib,"bassenc.lib") // for recording
#pragma comment(lib,"rpcrt4.lib") // for GUID generation
#endif


#define MAX_STATION_COUNT 2048

char proxy[100]=""; // proxy server

int currentStationIndex=0;

char lame_options[128];
char path[MAX_PATH];
HMENU* popupMenus;
int popupCount=1;
char **urlList;
int urlCount=0;
BOOL iswma;

HWND win=NULL;
CRITICAL_SECTION lock;
DWORD req=0;	// request number/counter
HSTREAM chan=0;	// stream handle
HENCODE encHandle=0;

const char* WND_TITLE="Light Radio";
const char* WND_TITLE_REC="Light Radio  [REC]";

wchar_t* recordFilePath=0;

#define MESS(id,m,w,l) SendDlgItemMessageA(win,id,m,(WPARAM)(w),(LPARAM)(l))

wchar_t* ConToUnicode(char* text)
{
	int nLen = MultiByteToWideChar(CP_UTF8, 0,text, -1, NULL, 0);
	wchar_t* lpszW = malloc(sizeof(wchar_t)*nLen);
	MultiByteToWideChar(CP_UTF8, 0,text, -1, lpszW, nLen);
	return lpszW;
}


// display error messages
void Error(const char *es)
{
	char mes[200];
	sprintf(mes,"%s\n(Error Code: %d)",es,BASS_ErrorGetCode());
	MessageBoxA(win,mes,"Error",MB_ICONERROR);
}

// update stream title from metadata
void DoMeta()
{
	if (iswma) {
		const char *meta=BASS_ChannelGetTags(chan,BASS_TAG_WMA_META);
		if (meta) // got a script/mid-stream tag, display it (could check for a specific tag, eg. "caption")
		{
			//MESS(IDC_EDIT1,WM_SETTEXT,0,meta);
			wchar_t* ut=ConToUnicode((char*)meta);
			SendDlgItemMessageW(win,IDC_EDIT1,WM_SETTEXT,0,(LPARAM)ut);
			free(ut);
		}
	} else
	{	
		const char *meta=BASS_ChannelGetTags(chan,BASS_TAG_META);
		if (meta) { // got Shoutcast metadata
			const char *p=strstr(meta,"StreamTitle='"); // locate the title
			if (p) {
				const char *p2=strstr(p,"';"); // locate the end of it
				if (p2) {
					wchar_t* ut;
					char *t=_strdup(p+13);
					t[p2-(p+13)]=0;
					//MESS(IDC_EDIT1,WM_SETTEXT,0,t);
					ut=ConToUnicode(t);
					SendDlgItemMessageW(win,IDC_EDIT1,WM_SETTEXT,0,(LPARAM)ut);
					free(ut);

					free(t);
				}
			}
		} else {
			meta=BASS_ChannelGetTags(chan,BASS_TAG_OGG);
			if (meta) { // got Icecast/OGG tags
				const char *artist=NULL,*title=NULL,*p=meta;
				for (;*p;p+=strlen(p)+1) {
					if (!_strnicmp(p,"artist=",7)) // found the artist
						artist=p+7;
					if (!_strnicmp(p,"title=",6)) // found the title
						title=p+6;
				}
				if (title) {
					if (artist) {
						wchar_t* ut;
						char text[100];
						_snprintf(text,sizeof(text),"%s - %s",artist,title);
						//MESS(IDC_EDIT1,WM_SETTEXT,0,text);
						ut=ConToUnicode(text);
						SendDlgItemMessageW(win,IDC_EDIT1,WM_SETTEXT,0,(LPARAM)ut);
						free(ut);
					} else
					{
						//MESS(IDC_EDIT1,WM_SETTEXT,0,title);
						wchar_t* ut=ConToUnicode((char*)title);
						SendDlgItemMessageW(win,IDC_EDIT1,WM_SETTEXT,0,(LPARAM)ut);
						free(ut);

					}
				}
			}
		}
	}
}

void CALLBACK MetaSync(HSYNC handle, DWORD channel, DWORD data, void *user)
{
	DoMeta();
}

void CALLBACK WMAChangeSync(HSYNC handle, DWORD channel, DWORD data, void *user)
{
	// a new track (and perhaps new tags) in a server-side playlist
	const char *tags=BASS_ChannelGetTags(chan,BASS_TAG_WMA);
	if (tags) {
		for (;*tags;tags+=strlen(tags)+1) {
			if (!_strnicmp(tags,"title=",6))
			{
				//MESS(IDC_EDIT2,WM_SETTEXT,0,tags+6);
				wchar_t* ut=ConToUnicode((char*)(tags+6));
				SendDlgItemMessageW(win,IDC_EDIT2,WM_SETTEXT,0,(LPARAM)ut);
				free(ut);
			}
			if (!_strnicmp(tags,"author=",7))
			{
				//MESS(IDC_EDIT3,WM_SETTEXT,0,tags+7);
				wchar_t* ut=ConToUnicode((char*)(tags+7));
				SendDlgItemMessageW(win,IDC_EDIT3,WM_SETTEXT,0,(LPARAM)ut);
				free(ut);
			}
		}
	}
}

void CALLBACK EndSync(HSYNC handle, DWORD channel, DWORD data, void *user)
{
	MESS(IDC_EDIT2,WM_SETTEXT,0,"not playing");
	MESS(IDC_EDIT1,WM_SETTEXT,0,"");
	MESS(IDC_EDIT3,WM_SETTEXT,0,"");
}

void CALLBACK StatusProc(const void *buffer, DWORD length, void *user)
{
	if (buffer && !length && (DWORD)user==req) // got HTTP/ICY tags, and this is still the current request
		MESS(IDC_EDIT3,WM_SETTEXT,0,buffer); // display status
}



void __cdecl OpenURL(void *url)
{
	DWORD c,r;
	EnterCriticalSection(&lock); // make sure only 1 thread at a time can do the following
	r=++req; // increment the request counter for this request
	LeaveCriticalSection(&lock);
	KillTimer(win,0); // stop prebuffer monitoring
	BASS_StreamFree(chan); // close old stream
	MESS(IDC_EDIT2,WM_SETTEXT,0,"connecting...");
	MESS(IDC_EDIT1,WM_SETTEXT,0,"");
	MESS(IDC_EDIT3,WM_SETTEXT,0,"");
	c=BASS_StreamCreateURL((char*)url,0,BASS_STREAM_BLOCK|BASS_STREAM_STATUS|BASS_STREAM_AUTOFREE,StatusProc,(void*)r); // open URL
	free(url); // free temp URL buffer
	EnterCriticalSection(&lock);
	if (r!=req) { // there is a newer request, discard this stream
		LeaveCriticalSection(&lock);
		if (c) BASS_StreamFree(c);
		return;
	}
	chan=c; // this is now the current stream
	LeaveCriticalSection(&lock);
	if (!chan) { // failed to open
		MESS(IDC_EDIT2,WM_SETTEXT,0,"not playing");
		Error("Can't play the stream");
	} else
	{
		iswma=!!BASS_ChannelGetTags(chan,BASS_TAG_WMA); // check whether it's a WMA stream
		BASS_ChannelFlags(chan,BASS_STREAM_RESTRATE,BASS_STREAM_RESTRATE); // restrict the download rate.
		SetTimer(win,0,50,0); // start prebuffer monitoring		
	}
}

void MakeRecordNameRandom()
{
	// create UUID
	UUID uuid;
	wchar_t* str;
	UuidCreate(&uuid);
	UuidToStringW(&uuid, (wchar_t**)&str);
	wcscpy(recordFilePath,str);
	RpcStringFreeW((wchar_t**)&str);
}

// defines from bass_aac.h
#define BASS_CTYPE_STREAM_AAC	0x10b00 // AAC
#define BASS_CTYPE_STREAM_MP4	0x10b01 // MP4



BOOL CALLBACK dialogproc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch (msg) {

		case WM_TIMER:
			{ 
				// monitor prebuffering progress
				DWORD progress;
				if (iswma) // check the WMA buffering progress
				{ 
					progress=(DWORD)BASS_StreamGetFilePosition(chan,BASS_FILEPOS_WMA_BUFFER);
				}else
				{
					progress=(DWORD)BASS_StreamGetFilePosition(chan,BASS_FILEPOS_BUFFER)*100/BASS_StreamGetFilePosition(chan,BASS_FILEPOS_END); // percentage of buffer filled
				}
				
				if (progress>75 || !BASS_StreamGetFilePosition(chan,BASS_FILEPOS_CONNECTED)) // over 75% full (or end of download)
				{ 
					KillTimer(win,0); // finished prebuffering, stop monitoring
					{
						DWORD len;
						int bitrate;
						char* buff;
						BASS_CHANNELINFO info;
						

						if (iswma)
						{
							const char *tags=BASS_ChannelGetTags(chan,BASS_TAG_WMA);
							for (;*tags;tags+=strlen(tags)+1) {
								if (!_strnicmp(tags,"title=",6))
								{
									//MESS(IDC_EDIT2,WM_SETTEXT,0,tags+6);
									wchar_t* ut=ConToUnicode((char*)(tags+6));
									SendDlgItemMessageW(win,IDC_EDIT2,WM_SETTEXT,0,(LPARAM)ut);
									free(ut);
								}
								if (!_strnicmp(tags,"author=",7))
								{
									//MESS(IDC_EDIT3,WM_SETTEXT,0,tags+7);
									wchar_t* ut=ConToUnicode((char*)(tags+7));
									SendDlgItemMessageW(win,IDC_EDIT3,WM_SETTEXT,0,(LPARAM)ut);
									free(ut);
								}
							}
							// set syncs for title changes
							BASS_ChannelSetSync(chan,BASS_SYNC_WMA_META,0,&MetaSync,0); // script/mid-stream tags
							BASS_ChannelSetSync(chan,BASS_SYNC_WMA_CHANGE,0,&WMAChangeSync,0); // server-side playlist changes		
							
							MESS(IDC_STATIC1,WM_SETTEXT,0,"WMA");					
						}else
						{
							// get the broadcast name and URL
							const char *icy=BASS_ChannelGetTags(chan,BASS_TAG_ICY);
							if (!icy) icy=BASS_ChannelGetTags(chan,BASS_TAG_HTTP); // no ICY tags, try HTTP
							if (icy) {
								for (;*icy;icy+=strlen(icy)+1) {
									if (!_strnicmp(icy,"icy-name:",9))
									{
										//MESS(IDC_EDIT2,WM_SETTEXT,0,icy+9);
										wchar_t* ut=ConToUnicode((char*)(icy+9));
										SendDlgItemMessageW(win,IDC_EDIT2,WM_SETTEXT,0,(LPARAM)ut);
										free(ut);
									}
									if (!_strnicmp(icy,"icy-url:",8))
									{
										//MESS(IDC_EDIT3,WM_SETTEXT,0,icy+8);
										wchar_t* ut=ConToUnicode((char*)(icy+8));
										SendDlgItemMessageW(win,IDC_EDIT3,WM_SETTEXT,0,(LPARAM)ut);
										free(ut);
									}
								}
							} else
							{
								MESS(IDC_EDIT2,WM_SETTEXT,0,"");
							}
							
							BASS_ChannelSetSync(chan,BASS_SYNC_META,0,&MetaSync,0); // Shoutcast
							BASS_ChannelSetSync(chan,BASS_SYNC_OGG_CHANGE,0,&MetaSync,0); // Icecast/OGG
							
							// set bitrate label
							len=(DWORD)BASS_StreamGetFilePosition(chan, BASS_FILEPOS_END); // file length
							bitrate=(len*8)/BASS_GetConfig(BASS_CONFIG_NET_BUFFER); // bitrate (Kbps)
							buff=(char*)malloc(16);
							_itoa((int)bitrate,buff,10);
							strcat(buff," Kbps");
							

							// display type of stream
							BASS_ChannelGetInfo(chan, &info); // get info
							if(info.ctype==BASS_CTYPE_STREAM_MP3)
							{
								strcat(buff," MP3");
							}else if(info.ctype==BASS_CTYPE_STREAM_MP2)
							{
								strcat(buff," MP2");
							}else if(info.ctype==BASS_CTYPE_STREAM_MP1)
							{
								strcat(buff," MP1");
							}else if(info.ctype==BASS_CTYPE_STREAM_OGG)
							{
								strcat(buff," Ogg");
							}else if(info.ctype==BASS_CTYPE_STREAM_AAC)
							{
								strcat(buff," AAC");
							}else if(info.ctype==BASS_CTYPE_STREAM_MP4)
							{
								strcat(buff," MP4");
							}else
							{
								strcat(buff," ?");
							}

							MESS(IDC_STATIC1,WM_SETTEXT,0,buff);
							free(buff);
																				
						}
						
						// get the stream title and set sync for subsequent titles
						DoMeta();
						// set sync for end of stream
						BASS_ChannelSetSync(chan,BASS_SYNC_END,0,&EndSync,0);
						// play it!
						BASS_ChannelPlay(chan,FALSE);											
					}

				} else {
					char text[20];
					sprintf(text,"buffering... %d%%",progress);
					MESS(IDC_EDIT2,WM_SETTEXT,0,text);
				}
			}
			break;

		case WM_HSCROLL:
			if(GetDlgItem(hwnd,IDC_SLIDER1)==(HWND) lParam){
				DWORD dwPos;
				dwPos = (DWORD)SendMessage(GetDlgItem(hwnd,IDC_SLIDER1), TBM_GETPOS, 0, 0);
				BASS_SetConfig(BASS_CONFIG_GVOL_STREAM,(DWORD)((dwPos/(float)100)*10000));
			}
			break;

			/*case WM_RBUTTONDOWN:
			{
				POINT p;
				GetCursorPos(&p);
				SetForegroundWindow(hwnd);
				TrackPopupMenu(popupMenu,TPM_LEFTALIGN|TPM_TOPALIGN,p.x,p.y,0,hwnd,0);
			}
			break;*/

		case WM_COMMAND:
			switch (LOWORD(wParam)) {

				case IDC_BUTTON3: // about
					{
						SetFocus(hwnd); // remove ugly focus rectangle
						MessageBoxA(hwnd,"Light Radio v2.4\nLightweight Internet Radio For Windows\n\nSupports MP3, WMA, ASX, AAC, AAC+ & Ogg Vorbis Streams (SHOUTcast, Icecast etc...)\nCompiled using MinGW GCC v4.7.1\n\nVisit http://www.crownsoft.net","About",MB_ICONINFORMATION);

					}
					break;

				case IDC_BUTTON1: // stop
					{
						DWORD status;
						SetFocus(hwnd); // remove ugly focus rectangle

						// stop any recordings
						status=BASS_Encode_IsActive(encHandle);
						if( (status==BASS_ACTIVE_PLAYING) || (status==BASS_ACTIVE_PAUSED) )
						{
							BASS_Encode_Stop(encHandle);
							MoveFileW(L"rec.mp3",recordFilePath); // rename file
							SendMessageW(GetDlgItem(hwnd,IDC_CHECK1),BM_SETCHECK,0,0); // uncheck record button
							SetWindowTextA(hwnd,WND_TITLE);
						}

						BASS_StreamFree(chan);
					}
					return 1;
				case IDC_BUTTON2: // play
					{
						if(currentStationIndex!=0) // user already selected a channel, so play it
						{
							DWORD status;
							char* url;
							SetFocus(hwnd); // remove ugly focus rectangle

							// stop any recordings
							status=BASS_Encode_IsActive(encHandle);
							if( (status==BASS_ACTIVE_PLAYING) || (status==BASS_ACTIVE_PAUSED) )
							{
								BASS_Encode_Stop(encHandle);
								MoveFileW(L"rec.mp3",recordFilePath); // rename file
								SendMessageW(GetDlgItem(hwnd,IDC_CHECK1),BM_SETCHECK,0,0); // uncheck record button
								SetWindowTextA(hwnd,WND_TITLE);
							}


							CheckMenuItem(popupMenus[0],currentStationIndex+2000,MF_CHECKED);
							url=urlList[currentStationIndex];
							_beginthread(OpenURL,0,_strdup(url));
						}
						
					}
					return 1;
				case IDC_BUTTON4: // stations
					{
						RECT rect;
						SetFocus(hwnd); // remove ugly focus rectangle
						GetWindowRect(GetDlgItem(hwnd,IDC_BUTTON4),&rect);
						SetForegroundWindow(hwnd);
						TrackPopupMenu(popupMenus[0],TPM_LEFTALIGN|TPM_TOPALIGN,rect.left,rect.bottom,0,hwnd,0);
						return 1;
					}
				case IDC_CHECK1: // record
					{
						BASS_CHANNELINFO chInf;
						SetFocus(hwnd); // remove ugly focus rectangle

						if(BASS_ChannelGetInfo(chan,&chInf)) // channel is playing
						{
							DWORD status=BASS_Encode_IsActive(encHandle);
							if( (status==BASS_ACTIVE_PLAYING) || (status==BASS_ACTIVE_PAUSED) )
							{
								BASS_Encode_Stop(encHandle);
								MoveFileW(L"rec.mp3",recordFilePath); // rename file
								SetWindowTextA(hwnd,WND_TITLE);
							}else
							{
								char *finalarg;

								GetWindowTextW(GetDlgItem(hwnd,IDC_EDIT1),recordFilePath,MAX_PATH);

								if(wcslen(recordFilePath)!=0)
								{
									// remove invalid characters using a shell function
									PathCleanupSpec(NULL,recordFilePath);
								}else // title is empty
								{
									MakeRecordNameRandom();
								}

								wcscat(recordFilePath,L".mp3");

								if(GetFileAttributesW(recordFilePath)!=INVALID_FILE_ATTRIBUTES) // file exists, so lets create a guid
								{
									MakeRecordNameRandom();
									wcscat(recordFilePath,L".mp3");
								}

								// lame commandline
								finalarg=(char*)malloc(MAX_PATH);
								strcpy(finalarg,"lame.exe ");	
								strcat(finalarg,lame_options);
								strcat(finalarg," - \"rec.mp3\"");

								encHandle=BASS_Encode_Start(chan,finalarg, 0, NULL, 0);
								SetWindowTextA(hwnd,WND_TITLE_REC);
								free(finalarg);
							}
						}else // nothing is playing
						{
							SendMessageW(GetDlgItem(hwnd,IDC_CHECK1),BM_SETCHECK,0,0); // uncheck record button
						}
					}
					return 1;
			   default:
					if ( (LOWORD(wParam)>=2000) && (LOWORD(wParam)<(2000+MAX_STATION_COUNT)) ) // it's menu item
					{
						int i,nullCount;
						char* url;

						// stop any recordings
						DWORD status=BASS_Encode_IsActive(encHandle);
						if( (status==BASS_ACTIVE_PLAYING) || (status==BASS_ACTIVE_PAUSED) )
						{
							BASS_Encode_Stop(encHandle);
							MoveFileW(L"rec.mp3",recordFilePath); // rename file
							SendMessageW(GetDlgItem(hwnd,IDC_CHECK1),BM_SETCHECK,0,0); // uncheck record button
							SetWindowTextA(hwnd,WND_TITLE);
						}

						CheckMenuItem(popupMenus[0],currentStationIndex+2000,MF_UNCHECKED); // uncheck old one
						// uncheck old category
						nullCount=-1;
						for(i=currentStationIndex;i>=0;i--)
						{
							if(strcmp(urlList[i],"null")==0)
							{
								nullCount++;
							}
						}
						if(nullCount==-1)
							nullCount=0;
						CheckMenuItem(popupMenus[0],nullCount,MF_UNCHECKED|MF_BYPOSITION);

						currentStationIndex=LOWORD(wParam)-2000;
						CheckMenuItem(popupMenus[0],LOWORD(wParam),MF_CHECKED); // check new one

						// check category
						nullCount=-1;
						for(i=currentStationIndex;i>=0;i--)
						{
							if(strcmp(urlList[i],"null")==0)
							{
								nullCount++;
							}
						}
						if(nullCount==-1)
							nullCount=0;
						CheckMenuItem(popupMenus[0],nullCount,MF_CHECKED|MF_BYPOSITION);

						url=urlList[currentStationIndex];
						_beginthread(OpenURL,0,_strdup(url));
						return 1;
					}
			}
			break;

		case WM_INITDIALOG:
			{
				RECT rect;
				char buff[200],buff2[8];
				int stationCount;
				int i;
				HMENU currentPopupMenu;

				win=hwnd;

				SetClassLongA(hwnd,GCL_HICON,(LONG)LoadIconA(GetModuleHandle(0),MAKEINTRESOURCEA(IDI_ICON1)));

				GetWindowRect(hwnd,&rect);
				SetWindowPos(hwnd,HWND_NOTOPMOST,rect.left,150,0,0,SWP_NOSIZE);

				popupMenus[0]=CreatePopupMenu();
				currentPopupMenu=popupMenus[0];

				// load station list
				
				stationCount=GetPrivateProfileIntA("stations","station_count",0,path);
				for(i=0;i<stationCount;i++,urlCount++)
				{
					_itoa(i+1,buff2,10);				

					//get url
					strcpy(buff,"station_url_");
					strcat(buff,buff2);
					GetPrivateProfileStringA("stations",buff,"null",buff,200,path);
					
					urlList[i]=_strdup(buff); // even buff contain url or not we put it to list to sync with menu id
					
					if(strcmp(buff,"null")==0) // category
					{		
						// create new popup menu
						popupCount++;
						popupMenus[popupCount-1]=CreatePopupMenu();
						currentPopupMenu=popupMenus[popupCount-1];
						
						// get station name
						strcpy(buff,"station_name_");
						strcat(buff,buff2);
						GetPrivateProfileStringA("stations",buff,"null",buff,200,path);
						
						// add to popup menu here...				
						InsertMenuA(popupMenus[0], 0xFFFFFFFF, MF_BYPOSITION | MF_STRING | MF_POPUP , (UINT_PTR)currentPopupMenu, buff);							
					
					}else // station url
					{				
						// get station name
						strcpy(buff,"station_name_");
						strcat(buff,buff2);
						GetPrivateProfileStringA("stations",buff,"null",buff,200,path);
						
						// add to popup menu here...				
						InsertMenuA(currentPopupMenu, 0xFFFFFFFF, MF_BYPOSITION | MF_STRING, 2000+i, buff);								
					}
								
				}

				SendMessage(GetDlgItem(hwnd,IDC_SLIDER1), TBM_SETRANGE,(WPARAM) TRUE,(LPARAM) MAKELONG(0, 100));  
				SendMessage(GetDlgItem(hwnd,IDC_SLIDER1), TBM_SETPOS,(WPARAM) TRUE,(LPARAM)50);	

				// initialize default output device
				BASS_Init(-1,44100,0,hwnd,NULL);
				BASS_SetConfig(BASS_CONFIG_GVOL_STREAM,5000);

				BASS_PluginLoad("bass_aac.dll",0); // aac, aac+ support
				BASS_PluginLoad("bassflac.dll",0); // ogg vorbis support
				BASS_PluginLoad("basswma.dll",0); // wma, asf  support

				BASS_SetConfig(BASS_CONFIG_NET_PLAYLIST,1); // enable playlist processing
				BASS_SetConfig(BASS_CONFIG_NET_PREBUF,0); // minimize automatic pre-buffering, so we can do it (and display it) instead
				BASS_SetConfig(BASS_CONFIG_NET_TIMEOUT,10000); // 10 seconds timeout
				
				GetPrivateProfileStringA("config","use_proxy","off",buff,200,path);
				if(strcmp(buff,"on")==0)
				{
					GetPrivateProfileStringA("config","proxy","null",proxy,200,path);
					BASS_SetConfigPtr(BASS_CONFIG_NET_PROXY,proxy); // setup proxy server location
				}

				GetPrivateProfileStringA("config","lame_options","-b 128",lame_options,128,path);
				
				InitializeCriticalSection(&lock);
			}
			return 0;

		case WM_CLOSE:
			{
				// stop recording
				DWORD status=BASS_Encode_IsActive(encHandle);
				if( (status==BASS_ACTIVE_PLAYING) || (status==BASS_ACTIVE_PAUSED) )
				{
					BASS_Encode_Stop(encHandle);
					MoveFileW(L"rec.mp3",recordFilePath); // rename file
				}

				EndDialog(hwnd,0);
			}
			break;

		case WM_DESTROY:
			DeleteCriticalSection(&lock);
			BASS_Free();
			break;
	}
	return 0;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,LPSTR lpCmdLine, int nCmdShow)
{
	char *p;
	int i;

	urlList=(char**)malloc(4*MAX_STATION_COUNT);
	recordFilePath=(wchar_t*)malloc(sizeof(wchar_t)*MAX_PATH);
	popupMenus=(HMENU*)malloc(4*64); // max 64-1 station categories (-1 for main popup menu)
	

	// setup path...
	GetModuleFileNameA(hInstance,path,MAX_PATH);

	for (p = path; *p; p++) { }	// find end
	for (; p > path && *p != '\\'; p--) { } // back up to last backslash
	*p = 0;	// kill it

	strcat(path,"\\stations.ini");

	InitCommonControls();

	// check the correct BASS was loaded
	if (HIWORD(BASS_GetVersion())!=BASSVERSION) {
		MessageBoxW(0,L"An incorrect version of BASS.DLL was loaded",0,MB_ICONERROR);
		return 0;
	}

	// display the window
	DialogBoxW(hInstance,MAKEINTRESOURCEW(IDD_DIALOG1),0,dialogproc);


	// free urls
	for(i=0;i<urlCount;i++)
		free(urlList[i]);

	free(urlList);
	free(recordFilePath);
	
	for(i=0;i<popupCount;i++)
		DestroyMenu(popupMenus[i]);
		
	free(popupMenus);

	return 0;
}
