


#include <stdio.h>
#include <windows.h>
#include "resource.h"


char path[MAX_PATH];
HWND hListBox;

typedef struct RadioStation
{
	struct RadioStation *prev;
	struct RadioStation *next;
	char *url;
	char *name;
	int categoryFlag;	 
}RadioStation;

RadioStation *firstStation; // first item of double linked list

int GetListBoxItemCount()
{
	int count=(int)SendMessageW(hListBox,LB_GETCOUNT,0,0);
	if(count!=LB_ERR)
		return count;
	return -1;	
}

int GetListBoxSelectedItem()
{
	int index=(int)SendMessageW(hListBox,LB_GETCURSEL,0,0);
	if(index!=LB_ERR)
		return index;
	return -1;		
}

RadioStation* GetStationByIndex(int index)
{
	RadioStation *tmp;
	tmp=firstStation;
	
	if(firstStation)
	{
		int i=0;
		while(tmp)
		{
			if(i==index)
				return tmp;
			tmp=tmp->next;
			i++;
		}
		return 0;
	}else
	{
		return 0;
	}
}

void SelectItemInListBox(int index,int scrollPos)
{
	SendMessageW(hListBox,LB_SETCURSEL,index,0);
	if(scrollPos!=-1)
		SendMessageA(hListBox,WM_VSCROLL,MAKEWPARAM(SB_THUMBPOSITION,scrollPos),0);
}

int UpdateListBox()
{
	if(firstStation)
	{
		RadioStation *tmp;
		char buff[200];
		int scrollPos;
		
		scrollPos=GetScrollPos(hListBox,SB_VERT);
		
		
		SendMessageA(hListBox,LB_RESETCONTENT,0,0); // clear listbox
		tmp=firstStation;
		while(tmp)
		{
			// add to listbox
			if(tmp->categoryFlag) // category
			{
				strcpy(buff,"  [+]  ");
				strcat(buff,tmp->name);
				SendMessageA(hListBox,LB_ADDSTRING,0,(LPARAM)buff);
			}else // station name
			{
				strcpy(buff,"               ");
				strcat(buff,tmp->name);				
				SendMessageA(hListBox,LB_ADDSTRING,0,(LPARAM)buff);
			}
							
			tmp=tmp->next;
		}
		
		return scrollPos;
		
	}else
	{
		SendMessageA(hListBox,LB_RESETCONTENT,0,0); // clear listbox
	}
	return 0;
}

//Split song length and title:
int splitSongInfo(char *text, char *chrLength, char *title){
	strcpy(chrLength, "");
	strcpy(title, "");
	if (strstr(text, ",") != NULL){
		int i = 0;
		int posSep = -1;
		int countLength = 0;
		int countTitle = 0;

		for (i = 8; i < strlen(text); i++){
			if ((int)text[i] == 10 || (int)text[i] == 13){
				continue;
			}
			if (text[i] == ','){
				posSep = i;
			}else if (posSep == -1){
				if ((countLength) | (text[i] != ' ')){
					chrLength[countLength] = text[i];
					countLength++;
				}
			}else{
				if ((countTitle) | (text[i] != ' ')){
					title[countTitle] = text[i];
					countTitle++;
				}
			}
		}
		chrLength[countLength] = '\0';
		title[countTitle] = '\0';
		return(1);
	}
	return(0);
}

int ReadEXTM3U(char* path)
{
	FILE *f;
	char lineText[512];
	char chrLength[20];
	char title[264];
	int foundINF;

	int index=GetListBoxSelectedItem();

	f = fopen(path, "rt");
	if (f == NULL){
		return -1;
	}

	foundINF=0;
	while(fgets(lineText, 256, f) != NULL)
	{
		if (!strncmp(lineText, "#EXTINF:", 8))
		{
			foundINF=1;
			chrLength[0]=0;
			title[0]=0;

			//Length and title
			splitSongInfo(lineText, chrLength, title);
		}else if (!strncmp(lineText, "#EXTM3U", 7))
		{
			//Nothing to do. :)
		}else if (strlen(lineText) > 2)
		{
			if(foundINF)
			{
				if(strlen(title)>1) // read urls which has title
				{

					RadioStation *station;
					station=(RadioStation*)malloc(sizeof(RadioStation));

					station->url=_strdup(lineText);
					station->name=_strdup(title);

					if(strcmp(station->url,"null")==0)
					{
						station->categoryFlag=1;
					}else
					{
						station->categoryFlag=0;
					}

					if(index!=-1) // add after item index
					{
						RadioStation *lastStation;
						lastStation=GetStationByIndex(index);

						station->next=lastStation->next;
						station->prev=lastStation;

						lastStation->next=station;

						if(station->next)
							station->next->prev=station;
													
					}else // add to last
					{
						RadioStation *tmp,*last;

						tmp=firstStation;

						if(tmp) // we have atleast one item
						{

							while(tmp) // move to last
							{
								if(tmp->next==0)
									last=tmp;
								tmp=tmp->next;
							}

							last->next=station;
							station->prev=last;
							station->next=0;								
						}else // list is empty
						{
							station->prev=0;
							station->next=0;
							firstStation=station;															
						}						
					}
				}
				foundINF=0;
			}
		}
	}

	SelectItemInListBox(index+1,UpdateListBox());
	fclose(f);
	return 0;
}

// checks for existence of #EXTM3U within a file
int IsEXTM3U(char* path)
{
	char lineText[512];
	FILE *f=fopen(path, "rt");
	if (f == NULL){
		return 0;
	}

	while(fgets(lineText, 256, f) != NULL)
	{
		if (!strncmp(lineText, "#EXTM3U", 7))
		{
			fclose(f);
			return 1;
		}
	}

	fclose(f);
	return 0;
}


BOOL CALLBACK DialogProc(
  HWND hwndDlg,  // handle to dialog box
  UINT uMsg,     // message
  WPARAM wParam, // first message parameter
  LPARAM lParam  // second message parameter
)
{
	switch(uMsg)
	{
		case WM_INITDIALOG:
		{
			int i;
			int stationCount;
			char buff[200],buff2[8];
			RadioStation *prevStation,*station;
			
			prevStation=0;
			station=0;
			
			hListBox=GetDlgItem(hwndDlg,IDC_LIST1);
			
			stationCount=GetPrivateProfileIntA("stations","station_count",0,path);
			for(i=0;i<stationCount;i++)
			{
				station=(RadioStation*)malloc(sizeof(RadioStation));
				_itoa(i+1,buff2,10);
				
				// get station name
				strcpy(buff,"station_name_");
				strcat(buff,buff2);
				GetPrivateProfileStringA("stations",buff,"null",buff,200,path);
				
				station->name=_strdup(buff);
				
				//get url
				strcpy(buff,"station_url_");
				strcat(buff,buff2);
				GetPrivateProfileStringA("stations",buff,"null",buff,200,path);
				
				station->url=_strdup(buff);
				
				if(strcmp(buff,"null")==0)
				{
					station->categoryFlag=1;
				}else
				{
					station->categoryFlag=0;
				}
				
				if(i==0) // first one
				{
					station->prev=0;
					station->next=0; // there might be only one item
					firstStation=station;
				}else if(i==(stationCount-1)) // last one
				{
					station->next=0;
					station->prev=prevStation;
					if(prevStation)
						prevStation->next=station;
				}else 
				{
					station->prev=prevStation;
					prevStation->next=station;
				}
				
				prevStation=station;
				
				// add to listbox
				if(station->categoryFlag) // category
				{
					strcpy(buff,"  [+]  ");
					strcat(buff,station->name);
					SendMessageA(hListBox,LB_ADDSTRING,0,(LPARAM)buff);
				}else // station name
				{
					strcpy(buff,"               ");
					strcat(buff,station->name);				
					SendMessageA(hListBox,LB_ADDSTRING,0,(LPARAM)buff);
				}
								
			}

			return TRUE;
		}
		
		case WM_COMMAND:
		{
			if(LOWORD(wParam)==IDC_BUTTON1) // save button
			{
				int itemCount=GetListBoxItemCount();
				if(itemCount) 
				{
					int i;
					char buff[64],buff2[8],buff3[128];
					RadioStation *station;
															
					station=firstStation;
					
					// get config
					GetPrivateProfileStringA("config","proxy","null",buff,64,path);
					GetPrivateProfileStringA("config","use_proxy","null",buff2,8,path);
					GetPrivateProfileStringA("config","lame_options","-b 128",buff3,128,path);

					DeleteFileA(path); // delete old file
					
					// write config
					WritePrivateProfileStringA("config","use_proxy",buff2,path);
					WritePrivateProfileStringA("config","proxy",buff,path);	
					WritePrivateProfileStringA("config","lame_options",buff3,path);						
					
					// define station count
					_itoa(itemCount,buff2,10);
					WritePrivateProfileStringA("stations","station_count",buff2,path);
					
					for(i=1;i<=itemCount;i++)
					{
						_itoa(i,buff2,10);
					
						// write station name
						strcpy(buff,"station_name_");
						strcat(buff,buff2);
						WritePrivateProfileStringA("stations",buff,station->name,path);
						
						// write url
						strcpy(buff,"station_url_");
						strcat(buff,buff2);
						WritePrivateProfileStringA("stations",buff,station->url,path);
						
						station=station->next;						
					}
									
				}
				
				EndDialog(hwndDlg,0);
				return TRUE;
			}else if(LOWORD(wParam)==IDC_BUTTON5) // up button
			{
				if(GetListBoxItemCount()>1) // list contain more than 1 item
				{
					int index=GetListBoxSelectedItem();
					if(index>0) // item already selected & not the first one
					{
						int scrollPos;
						RadioStation *curStation,*prevStation;
						curStation=GetStationByIndex(index);
						prevStation=curStation->prev;
						
						curStation->prev=prevStation->prev;
						prevStation->next=curStation->next;
						
						if(curStation->next)
							curStation->next->prev=prevStation;
						
						if(prevStation->prev)
							prevStation->prev->next=curStation;
							
						curStation->next=prevStation;
						prevStation->prev=curStation;
						
						if(curStation->prev==0) // update firstStation variable
							firstStation=curStation;
							
						scrollPos=UpdateListBox();
						SelectItemInListBox(index-1,scrollPos);
						
					}
				}
			}else if(LOWORD(wParam)==IDC_BUTTON6) // down button
			{
				if(GetListBoxItemCount()>1) // list contain more than 1 item
				{
					int index=GetListBoxSelectedItem();
					if(index!=-1) // user already selected item
					{
						int scrollPos;
						RadioStation *curStation,*nextStation;
						curStation=GetStationByIndex(index);
						if(curStation->next!=0)	// not the last item
						{
							curStation=GetStationByIndex(index);
							nextStation=curStation->next;	
							
							curStation->next=nextStation->next;
							nextStation->prev=curStation->prev;	
							
							if(curStation->prev)
								curStation->prev->next=nextStation;
							
							if(nextStation->next)
								nextStation->next->prev=curStation;
								
							curStation->prev=nextStation;
							nextStation->next=curStation;	
							
							if(nextStation->prev==0) // update firstStation variable
								firstStation=nextStation;													
								
							scrollPos=UpdateListBox();
							SelectItemInListBox(index+1,scrollPos);																									
						}					
					}
				}				
			}else if(LOWORD(wParam)==IDC_BUTTON2) // add button
			{
				char buff[200];
				int index=GetListBoxSelectedItem();
				
				GetDlgItemTextA(hwndDlg,IDC_EDIT1,buff,200);
				if(strlen(buff)!=0)
				{
					GetDlgItemTextA(hwndDlg,IDC_EDIT2,buff,200);
					if(strlen(buff)!=0)
					{
						RadioStation *station;
						station=(RadioStation*)malloc(sizeof(RadioStation));
						
						station->url=_strdup(buff);
						GetDlgItemTextA(hwndDlg,IDC_EDIT1,buff,200);
						station->name=_strdup(buff);
						
						if(strcmp(station->url,"null")==0)
						{
							station->categoryFlag=1;
						}else
						{
							station->categoryFlag=0;
						}
						
						if(index!=-1) // add after item index
						{
							int scrollPos;
							RadioStation *lastStation;
							lastStation=GetStationByIndex(index);
							
							station->next=lastStation->next;
							station->prev=lastStation;
							
							lastStation->next=station;
	
							if(station->next)
								station->next->prev=station;
							
							scrollPos=UpdateListBox();
							SelectItemInListBox(index+1,scrollPos);														
						}else // add to last
						{
							int newItemIndex;
							RadioStation *tmp,*last;

							tmp=firstStation;
							newItemIndex=0;
							
							if(tmp) // we have atleast one item
							{
								
								while(tmp) // move to last
								{
									if(tmp->next==0)
										last=tmp;
									tmp=tmp->next;
									newItemIndex++;
								}
								
								last->next=station;
								station->prev=last;
								station->next=0;								
							}else // list is empty
							{
								station->prev=0;
								station->next=0;
								firstStation=station;															
							}						
							
							UpdateListBox();
							SelectItemInListBox(newItemIndex,-1);
						}
						
					}else // station url is empty
					{
						MessageBoxA(hwndDlg,"Station url is empty!","Error",MB_ICONERROR);
					}	
				}else // station name is empty
				{
					MessageBoxA(hwndDlg,"Station name is empty!","Error",MB_ICONERROR);
				}
			}else if(HIWORD(wParam)==LBN_SELCHANGE) // list box change
			{
				int index;
				RadioStation *station;
				
				index=GetListBoxSelectedItem();
				if(index!=-1)
				{
					station=GetStationByIndex(index);
					SetDlgItemTextA(hwndDlg,IDC_EDIT1,station->name);
					SetDlgItemTextA(hwndDlg,IDC_EDIT2,station->url);
				}
			}else if(LOWORD(wParam)==IDC_BUTTON3) // update button
			{
				int index;
				RadioStation *station;
				char buff[200];
				
				index=GetListBoxSelectedItem();
				if(index!=-1)
				{
					station=GetStationByIndex(index);
					GetDlgItemTextA(hwndDlg,IDC_EDIT1,buff,200);
					if(strlen(buff)!=0)
					{
						GetDlgItemTextA(hwndDlg,IDC_EDIT2,buff,200);
						if(strlen(buff)!=0)
						{	
							int scrollPos;
										
							free(station->name);
							free(station->url);
							
							station->url=_strdup(buff);
							GetDlgItemTextA(hwndDlg,IDC_EDIT1,buff,200);
							station->name=_strdup(buff);
							
							if(strcmp(station->url,"null")==0)
							{
								station->categoryFlag=1;
							}else
							{
								station->categoryFlag=0;
							}
							
							scrollPos=UpdateListBox();
							SelectItemInListBox(index,scrollPos);							
						}else
						{
							MessageBoxA(hwndDlg,"Station url is empty!","Error",MB_ICONERROR);
						}
					}else
					{
						MessageBoxA(hwndDlg,"Station name is empty!","Error",MB_ICONERROR);
					}
				}else
				{
					MessageBoxA(hwndDlg,"Please select a station!","Error",MB_ICONWARNING);
				}				
			}else if(LOWORD(wParam)==IDC_BUTTON4) // delete button
			{
				int index;
				
				index=GetListBoxSelectedItem();
				if(index!=-1)	
				{
					int scrollPos;
					RadioStation *station;
					station=GetStationByIndex(index);
					
					if(station->prev==0) // we are deleting first item
					{
						if(station->next)
						{
						  station->next->prev=0;
						  firstStation=station->next;
						}else
						{
							firstStation=0;
						}
					}else if(station->next==0) // we are deleting last item
					{
						station->prev->next=0;
					}else // we are deleting middle item
					{
						station->prev->next=station->next;
						station->next->prev=station->prev;
					}
					
					free(station->url);
					free(station->name);
					free(station);
					
					scrollPos=UpdateListBox();
					SelectItemInListBox(index,scrollPos);						
					
				}else
				{
					MessageBoxA(hwndDlg,"Please select a station!","Error",MB_ICONWARNING);
				}	
			}else if(LOWORD(wParam)==IDC_BUTTON7) // m3u import button
			{
				OPENFILENAMEA ofn;
				static char buff[MAX_PATH];

				ZeroMemory(&ofn, sizeof(OPENFILENAMEA));

				ofn.lStructSize = sizeof(OPENFILENAMEA);
				ofn.hwndOwner = hwndDlg;
				ofn.lpstrFilter = "Playlist (*.m3u)\0*.m3u\0";
				ofn.lpstrFile = buff;
				ofn.nMaxFile = MAX_PATH;
				ofn.Flags =  OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;
				ofn.lpstrTitle = "Import m3u playlist";

				if(GetOpenFileNameA(&ofn))
				{
					if(IsEXTM3U(buff))
					{
						ReadEXTM3U(buff);
					}else
					{
						MessageBoxA(hwndDlg,"Not an extended m3u file!","Error",MB_ICONWARNING);
					}
				}	
			}
			
			break;
		}
		
		case WM_CLOSE:
			EndDialog(hwndDlg,0);
			return TRUE;
			
		default:
			return FALSE;
	}
	
	return FALSE;
}

int WINAPI WinMain(
  HINSTANCE hInstance,  // handle to current instance
  HINSTANCE hPrevInstance,  // handle to previous instance
  LPSTR lpCmdLine,      // pointer to command line
  int nCmdShow          // show state of window
)
{
	char *p;
	RadioStation *tmp;

	firstStation=0;

	// setup path...
	GetModuleFileNameA(hInstance,path,MAX_PATH);

	for (p = path; *p; p++) { }	// find end
	for (; p > path && *p != '\\'; p--) { } // back up to last backslash
	*p = 0;	// kill it

	strcat(path,"\\stations.ini");
		
	DialogBox(hInstance,MAKEINTRESOURCE(IDD_DIALOG1),0,DialogProc);
	
	// free linked list
	while(firstStation)
	{
		tmp=firstStation;
		firstStation=tmp->next;
		
		free(tmp->url);
		free(tmp->name);
		free(tmp);		
		
	}
	
	return 0;
}
